import logging
from datetime import datetime

from .const import FanModes, ActivityNames
from .util import safely_get_json_value

_LOGGER = logging.getLogger(__name__)


def active_schedule_periods(periods_json: [dict]):
    return list(filter(lambda period: safely_get_json_value(period, "enabled") == "on", periods_json))


class ConfigZoneActivity:
    def __init__(self, zone_activity_json: dict):
        self.api_id: ActivityNames = ActivityNames(safely_get_json_value(zone_activity_json, "$.id"))
        self.fan: FanModes = FanModes(zone_activity_json["fan"])
        self.heat_set_point: float = safely_get_json_value(zone_activity_json, "htsp", float)
        self.cool_set_point: float = safely_get_json_value(zone_activity_json, "clsp", float)

    def __repr__(self):
        return {
            "api_id": self.api_id.value,
            "fan": self.fan.value,
            "heat_set_point": self.heat_set_point,
            "cool_set_point": self.cool_set_point,
        }

    def __str__(self):
        return str(self.__repr__())


class ConfigZone:
    def __init__(self, zone_json: dict, vacation_json: dict):
        self.api_id = safely_get_json_value(zone_json, "$.id")
        self.name: str = safely_get_json_value(zone_json, "name")
        self.hold_activity: ActivityNames = safely_get_json_value(zone_json, "holdActivity", ActivityNames)
        self.hold: bool = safely_get_json_value(zone_json, "hold") == "on"
        self.hold_until: str = safely_get_json_value(zone_json, "otmr")
        self.program_json: dict = safely_get_json_value(zone_json, "program")
        self.activities = []
        for zone_activity_json in safely_get_json_value(zone_json, "activities.activity"):
            self.activities.append(
                ConfigZoneActivity(zone_activity_json=zone_activity_json)
            )
        self.activities.append(ConfigZoneActivity(zone_activity_json=vacation_json))

    def find_activity(self, activity_name: ActivityNames):
        for activity in self.activities:
            if activity.api_id == activity_name:
                return activity

    def current_activity(self) -> ConfigZoneActivity:
        if self.hold:
            return self.find_activity(self.hold_activity)
        else:
            now = datetime.now()
            sunday_0_index_today = int(now.date().strftime("%w"))
            today_schedule_json = self.program_json["day"][sunday_0_index_today]
            active_periods = reversed(
                active_schedule_periods(today_schedule_json["period"])
            )
            for active_period in active_periods:
                hours, minutes = active_period["time"].split(":")
                if (int(hours) < now.hour) or (
                    int(hours) == now.hour and int(minutes) < now.minute
                ):
                    return self.find_activity(safely_get_json_value(active_period, "activity", ActivityNames))
            yesterday_schedule = self.program_json["day"][(sunday_0_index_today + 8) % 7]
            yesterday_active_periods = list(reversed(
                active_schedule_periods(yesterday_schedule["period"])
            ))
            return self.find_activity(safely_get_json_value(yesterday_active_periods[-1], "activity", ActivityNames))

    def next_activity_time(self) -> str:
        now = datetime.now()
        sunday_0_index_today = int(now.date().strftime("%w"))
        today_schedule_json = self.program_json["day"][sunday_0_index_today]
        active_periods = active_schedule_periods(today_schedule_json["period"])
        for active_period in active_periods:
            hours, minutes = active_period["time"].split(":")
            if (int(hours) > now.hour) or (
                int(hours) == now.hour and int(minutes) > now.minute
            ):
                return active_period["time"]
        tomorrow_schedule = self.program_json["day"][(sunday_0_index_today + 1) % 7]
        return active_schedule_periods(tomorrow_schedule["period"])[0]["time"]

    def __repr__(self):
        builder = {
            "api_id": self.api_id,
            "name": self.name,
            "hold_activity": self.hold_activity,
            "hold": self.hold,
            "hold_until": self.hold_until,
            "activities": list(map(lambda activity: activity.__repr__(), self.activities)),
        }
        if self.hold_activity is not None:
            builder["hold_activity"] = self.hold_activity.value
        return builder

    def __str__(self):
        return str(self.__repr__())


class Config:
    temperature_unit: str = None
    static_pressure: float = None
    mode: str = None
    limit_min: int = None
    limit_max: int = None
    zones: [ConfigZone] = None
    raw_config_json: dict = None

    def __init__(
        self,
        system,
    ):
        self.system = system
        self.refresh()

    def refresh(self):
        self.raw_config_json = self.system.api_connection.get_config(
            system_serial=self.system.serial
        )
        self.temperature_unit = safely_get_json_value(self.raw_config_json, "cfgem")
        self.static_pressure = safely_get_json_value(self.raw_config_json, "staticPressure")
        self.mode = safely_get_json_value(self.raw_config_json, "mode")
        self.limit_min = safely_get_json_value(self.raw_config_json, "utilityEvent.minLimit", int)
        self.limit_max = safely_get_json_value(self.raw_config_json, "utilityEvent.maxLimit", int)
        vacation_json = {
            "$": {"id": "vacation"},
            "clsp": self.raw_config_json["vacmaxt"],
            "htsp": self.raw_config_json["vacmint"],
            "fan": self.raw_config_json["vacfan"],
        }
        self.zones = []
        for zone_json in safely_get_json_value(self.raw_config_json, "zones.zone"):
            if safely_get_json_value(zone_json, "enabled") == "on":
                self.zones.append(
                    ConfigZone(zone_json=zone_json, vacation_json=vacation_json)
                )

    def __repr__(self):
        return {
            "temperature_unit": self.temperature_unit,
            "static_pressure": self.static_pressure,
            "mode": self.mode,
            "limit_min": self.limit_min,
            "limit_max": self.limit_max,
            "zones": list(map(lambda zone: zone.__repr__(), self.zones)),
        }

    def __str__(self):
        return str(self.__repr__())



import logging

from .const import SystemModes, TemperatureUnits, FanModes, ActivityNames
from .util import safely_get_json_value

_LOGGER = logging.getLogger(__name__)


class StatusZone:
    def __init__(self, status_zone_json: dict):
        self.api_id = safely_get_json_value(status_zone_json, "$.id")
        self.name: str = safely_get_json_value(status_zone_json, "name")
        self.current_activity: ActivityNames = ActivityNames(status_zone_json["currentActivity"])
        self.temperature: float = safely_get_json_value(status_zone_json, "rt", float)
        self.humidity: int = safely_get_json_value(status_zone_json, "rh", int)
        self.occupancy: bool = safely_get_json_value(status_zone_json, "occupancy") == "occupied"
        self.fan: FanModes = FanModes(status_zone_json["fan"])
        self.hold: bool = safely_get_json_value(status_zone_json, "hold") == "on"
        self.hold_until: str = safely_get_json_value(status_zone_json, "otmr")
        self.heat_set_point: float = safely_get_json_value(status_zone_json, "htsp", float)
        self.cool_set_point: float = safely_get_json_value(status_zone_json, "clsp", float)
        self.conditioning: str = safely_get_json_value(status_zone_json, "zoneconditioning")

    @property
    def zone_conditioning_const(self) -> str:
        match self.conditioning:
            case "active_heat" | "prep_heat" | "pending_heat":
                return SystemModes.HEAT
            case "active_cool" | "prep_cool" | "pending_cool":
                return SystemModes.COOL
            case "idle":
                return SystemModes.OFF

    def __repr__(self):
        return {
            "id": self.api_id,
            "name": self.name,
            "current_activity": self.current_activity.value,
            "temperature": self.temperature,
            "humidity": self.humidity,
            "fan": self.fan.value,
            "hold": self.hold,
            "occupancy": self.occupancy,
            "hold_until": self.hold_until,
            "heat_set_point": self.heat_set_point,
            "cool_set_point": self.cool_set_point,
            "conditioning": self.conditioning,
        }

    def __str__(self):
        return str(self.__repr__())


class Status:
    outdoor_temperature: int = None
    mode: str = None
    temperature_unit: str = None
    filter_used: int = None
    is_disconnected: bool = None
    airflow_cfm: int = None
    zones: [StatusZone] = None
    raw_status_json: dict = None

    def __init__(
        self,
        system,
    ):
        self.system = system
        self.refresh()

    def refresh(self):
        self.raw_status_json = self.system.api_connection.get_status(
            system_serial=self.system.serial
        )
        self.outdoor_temperature: float = safely_get_json_value(self.raw_status_json, "oat", float)
        self.mode: str = self.raw_status_json["mode"]
        self.temperature_unit: TemperatureUnits = TemperatureUnits(self.raw_status_json["cfgem"])
        self.filter_used: int = safely_get_json_value(self.raw_status_json, "filtrlvl", int)
        self.is_disconnected: bool = safely_get_json_value(self.raw_status_json, "isDisconnected", bool)
        self.airflow_cfm: int = safely_get_json_value(self.raw_status_json, "idu.cfm", int)
        self.zones = []
        for zone_json in self.raw_status_json["zones"]["zone"]:
            if safely_get_json_value(zone_json, "enabled") == "on":
                self.zones.append(StatusZone(zone_json))

    @property
    def mode_const(self) -> str:
        match self.mode:
            case "gasheat" | "electric" | "hpheat":
                return SystemModes.HEAT
            case "dehumidify":
                return SystemModes.COOL

    def __repr__(self):
        return {
            "outdoor_temperature": self.outdoor_temperature,
            "mode": self.mode,
            "temperature_unit": self.temperature_unit.value,
            "filter_used": self.filter_used,
            "is_disconnected": self.is_disconnected,
            "airflow_cfm": self.airflow_cfm,
            "zones": list(map(lambda zone: zone.__repr__(), self.zones)),
        }

    def __str__(self):
        return str(self.__repr__())



from enum import Enum

INFINITY_API_BASE_URL = "https://www.app-api.ing.carrier.com"
INFINITY_API_CONSUMER_KEY = "8j30j19aj103911h"
INFINITY_API_CONSUMER_SECRET = "0f5ur7d89sjv8d45"


class SystemModes(Enum):
    OFF = "off"
    COOL = "cool"
    HEAT = "heat"
    AUTO = "auto"
    FAN_ONLY = "fanonly"


class ActivityNames(Enum):
    HOME = "home"
    AWAY = "away"
    SLEEP = "sleep"
    WAKE = "wake"
    MANUAL = "manual"
    VACATION = "vacation"


class FanModes(Enum):
    OFF = "off"
    LOW = "low"
    MED = "med"
    HIGH = "high"


class TemperatureUnits(Enum):
    CELSIUS = "C"
    FAHRENHEIT = "F"



import logging

from .profile import Profile
from .status import Status
from .config import Config


_LOGGER = logging.getLogger(__name__)


class System:
    def __init__(
        self,
        api_connection,
        serial: str,
        name: str,
    ):
        self.api_connection = api_connection
        self.serial = serial
        self.name = name
        self.profile = Profile(system=self)
        self.status = Status(system=self)
        self.config = Config(system=self)

    def __repr__(self):
        return {
            "serial": self.serial,
            "name": self.name,
            "profile": self.profile.__repr__(),
            "status": self.status.__repr__(),
            "config": self.config.__repr__(),
        }

    def __str__(self):
        return str(self.__repr__())



import logging

_LOGGER = logging.getLogger(__name__)


def safely_get_json_value(json, key, callable_to_cast=None):
    value = json
    for x in key.split("."):
        if value is not None:
            try:
                value = value[x]
            except (TypeError, KeyError):
                try:
                    value = value[int(x)]
                except (TypeError, KeyError, ValueError):
                    value = None
    if callable_to_cast is not None and value is not None:
        value = callable_to_cast(value)
    return value



# run with "python3 src/carrier_api/stub.py"
import logging
import asyncio
from threading import Event

from getpass import getpass
from pathlib import Path
import sys

import asyncio

path_root = Path(__file__).parents[2]
sys.path.append(str(path_root))


logger = logging.getLogger("src.carrier_api")
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)
logger.addHandler(ch)

# logging.basicConfig()
# logging.getLogger().setLevel(logging.DEBUG)
# requests_log = logging.getLogger("requests.packages.urllib3")
# requests_log.setLevel(logging.DEBUG)
# requests_log.propagate = True


from src.carrier_api.api_connection import ApiConnection
from src.carrier_api.config import Config, ConfigZone, ConfigZoneActivity
from src.carrier_api.const import SystemModes, ActivityNames, FanModes


async def testing():
    username = input("username: ")
    password = getpass()
    connection = ApiConnection(username=username, password=password)
    system = connection.get_systems()[0]
    zone: ConfigZone = system.config.zones[0]
    logger.debug(system)
    # connection.set_config_hold(system_serial=system.serial, zone_id=zone.api_id, activity_name=ActivityNames.MANUAL, hold_until=None)
    logger.debug(zone.current_activity())
    # connection.resume_schedule(
    #     system_serial=system.serial, zone_id=system.config.zones[0].api_id
    # )
    # hold_until = zone.next_activity_time()
    # connection.set_config_mode(system_serial, SystemModes.HEAT.value)
    # connection.set_config_manual_activity(system_serial=system_serial, zone_id=zone.api_id, heat_set_point=71, cool_set_point=80, fan_mode=FanModes.LOW)


asyncio.run(testing())



import logging

import requests
from requests import Response
from requests_oauthlib import OAuth1
import xmltodict
from urllib.parse import quote

from .const import (
    INFINITY_API_BASE_URL,
    INFINITY_API_CONSUMER_KEY,
    INFINITY_API_CONSUMER_SECRET,
    FanModes,
    ActivityNames,
)
from .errors import AuthError
from .system import System

_LOGGER = logging.getLogger(__name__)


class ApiConnection:
    access_token = None

    def __init__(
        self,
        username: str,
        password: str,
    ):
        self.username = username
        self.password = password
        self.default_headers = {
            "featureset": "CONSUMER_PORTAL",
            "Accept": "application/json",
        }

    def _get(self, url: str) -> dict:
        oauth = OAuth1(
            INFINITY_API_CONSUMER_KEY,
            client_secret=INFINITY_API_CONSUMER_SECRET,
            resource_owner_key=self.username,
            resource_owner_secret=self._get_auth_token(),
            realm=url,
        )
        response = requests.get(url=url, auth=oauth, headers=self.default_headers)
        response.raise_for_status()
        return response.json()

    def _post(self, url: str, data: dict) -> Response:
        xml = xmltodict.unparse(data)
        data_xml = f"""data={quote(xml, safe="!~*'()")}"""
        if "users/authenticated" in url:
            resource_owner_secret = None
        else:
            resource_owner_secret = self._get_auth_token()
        oauth = OAuth1(
            INFINITY_API_CONSUMER_KEY,
            client_secret=INFINITY_API_CONSUMER_SECRET,
            resource_owner_key=self.username,
            resource_owner_secret=resource_owner_secret,
            realm=url,
        )
        response = requests.post(
            url=url, data=data_xml, auth=oauth, headers=self.default_headers
        )
        response.raise_for_status()
        return response

    def _get_auth_token(self) -> str:
        if self.access_token is None:
            url = f"{INFINITY_API_BASE_URL}/users/authenticated"
            creds = {
                "credentials": {
                    "username": self.username,
                    "password": self.password,
                },
            }
            response = self._post(url, data=creds)
            response_json = response.json()
            if "error" in response_json:
                _LOGGER.debug(response.content)
                raise AuthError(response_json["error"]["message"])
            self.access_token = response.json()["result"]["accessToken"]
        return self.access_token

    def get_systems(self) -> [System]:
        url = f"{INFINITY_API_BASE_URL}/users/{self.username}/locations"
        response_json = self._get(url)
        systems = []
        for location in response_json["locations"]["location"]:
            for system in location["systems"]["system"]:
                system_obj = system["atom:link"]["$"]
                systems.append(
                    System(
                        api_connection=self,
                        serial=system_obj["href"].split("/")[-1],
                        name=system_obj["title"],
                    )
                )
        return systems

    def get_profile(self, system_serial: str) -> dict:
        url = f"{INFINITY_API_BASE_URL}/systems/{system_serial}/profile"
        return self._get(url)["system_profile"]

    def get_status(self, system_serial: str) -> dict:
        url = f"{INFINITY_API_BASE_URL}/systems/{system_serial}/status"
        return self._get(url)["status"]

    def get_config(self, system_serial: str) -> dict:
        url = f"{INFINITY_API_BASE_URL}/systems/{system_serial}/config"
        return self._get(url)["config"]

    def update_config(self, system_serial: str, data: dict):
        url = f"{INFINITY_API_BASE_URL}/systems/{system_serial}/config"
        self._post(url=url, data=data)

    def set_config_mode(self, system_serial: str, mode: str):
        data = {"config": {"mode": mode}}
        self.update_config(system_serial=system_serial, data=data)

    def set_config_hold(
        self,
        system_serial: str,
        zone_id: str,
        activity_name: ActivityNames,
        hold_until: str = None,
    ):
        data = {
            "config": {
                "zones": {
                    "zone": [
                        {
                            "@id": zone_id,
                            "hold": "on",
                            "holdActivity": activity_name.value,
                            "otmr": hold_until,
                        }
                    ]
                }
            }
        }
        self.update_config(system_serial=system_serial, data=data)

    def resume_schedule(self, system_serial: str, zone_id: str):
        data = {
            "config": {
                "zones": {
                    "zone": [
                        {
                            "@id": zone_id,
                            "hold": "off",
                            "holdActivity": None,
                            "otmr": None,
                        }
                    ]
                }
            }
        }
        self.update_config(system_serial=system_serial, data=data)

    def set_config_manual_activity(
        self,
        system_serial: str,
        zone_id: str,
        heat_set_point: int,
        cool_set_point: int,
        fan_mode: FanModes,
    ):
        data = {
            "config": {
                "zones": {
                    "zone": [
                        {
                            "@id": zone_id,
                            "hold": "on",
                            "holdActivity": ActivityNames.MANUAL.value,
                            "activities": {
                                "activity": [
                                    {
                                        "@id": "manual",
                                        "htsp": float(heat_set_point),
                                        "clsp": float(cool_set_point),
                                        "fan": fan_mode.value,
                                    }
                                ]
                            },
                        }
                    ]
                }
            }
        }
        self.update_config(system_serial=system_serial, data=data)



import logging

from .util import safely_get_json_value

_LOGGER = logging.getLogger(__name__)


class Profile:
    model: str = None
    brand: str = None
    firmware: str = None
    indoor_model: str = None
    indoor_serial: str = None
    outdoor_model: str = None
    outdoor_serial: str = None
    zone_ids: [str] = None
    raw_profile_json: dict = None

    def __init__(
        self,
        system,
    ):
        self.system = system
        self.refresh()

    def refresh(self):
        self.raw_profile_json = self.system.api_connection.get_profile(
            system_serial=self.system.serial
        )
        self.model = safely_get_json_value(self.raw_profile_json, "model")
        self.brand = safely_get_json_value(self.raw_profile_json, "brand")
        self.firmware = safely_get_json_value(self.raw_profile_json, "firmware")
        self.indoor_model = safely_get_json_value(self.raw_profile_json, "indoorModel")
        self.indoor_serial = safely_get_json_value(self.raw_profile_json, "indoorSerial")
        self.outdoor_model = safely_get_json_value(self.raw_profile_json, "outdoorModel")
        self.outdoor_serial = self.raw_profile_json["outdoorSerial"]
        self.zone_ids = []
        for zone in safely_get_json_value(self.raw_profile_json, "zones.zone"):
            if safely_get_json_value(zone, "present") == "on":
                self.zone_ids.append(safely_get_json_value(zone, "$.id"))

    def __repr__(self):
        return {
            "model": self.model,
            "brand": self.brand,
            "firmware": self.firmware,
            "indoor_model": self.indoor_model,
            "indoor_serial": self.indoor_serial,
            "outdoor_model": self.outdoor_model,
            "outdoor_serial": self.outdoor_serial,
            "zone_ids": self.zone_ids,
        }

    def __str__(self):
        return str(self.__repr__())



from .errors import BaseError, RateError, AuthError
from .const import FanModes, ActivityNames, SystemModes, TemperatureUnits
from .api_connection import ApiConnection
from .config import Config, ConfigZone, ConfigZoneActivity
from .profile import Profile
from .status import Status, StatusZone
from .system import System
